package com.vaadin.spring.roo.addon.entityform;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.Validate;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.springframework.roo.addon.jpa.activerecord.JpaActiveRecordMetadata;
import org.springframework.roo.addon.web.mvc.controller.details.WebMetadataService;
import org.springframework.roo.classpath.PhysicalTypeMetadata;
import org.springframework.roo.classpath.TypeLocationService;
import org.springframework.roo.classpath.details.MethodMetadata;
import org.springframework.roo.classpath.itd.AbstractItdTypeDetailsProvidingMetadataItem;
import org.springframework.roo.classpath.scanner.MemberDetailsScanner;
import org.springframework.roo.metadata.MetadataService;
import org.springframework.roo.model.JavaSymbolName;
import org.springframework.roo.model.JavaType;

import com.vaadin.spring.roo.addon.entityview.AbstractVaadinEntityViewMethodBuilder.VaadinEntityViewMethodBuilderServices;
import com.vaadin.spring.roo.addon.entityview.DomainTypeCaptionMethodBuilder;
import com.vaadin.spring.roo.addon.entityview.PersistenceMethodBuilder;
import com.vaadin.spring.roo.addon.entityview.VaadinEntityMetadataDetails;

/**
 * Base class for EntityEditor metadata, both when using and when not using the
 * Vaadin Form component.
 */
public abstract class AbstractVaadinEntityFormMetadataItem extends
		AbstractItdTypeDetailsProvidingMetadataItem {

	private MetadataService metadataService;
	protected final MemberDetailsScanner memberDetailsScanner;
	private final WebMetadataService webMetadataService;
	private final TypeLocationService typeLocationService;

	private final VaadinEntityMetadataDetails vaadinEntityDetails;
	private boolean useJpaContainer;

	public AbstractVaadinEntityFormMetadataItem(String identifier,
			JavaType aspectName,
			PhysicalTypeMetadata governorPhysicalTypeMetadata,
			MetadataService metadataService,
			MemberDetailsScanner memberDetailsScanner,
			WebMetadataService webMetadataService,
			TypeLocationService typeLocationService,
			VaadinEntityMetadataDetails vaadinEntityDetails,
			boolean useJpaContainer) {
		super(identifier, aspectName, governorPhysicalTypeMetadata);

		Validate.notNull(metadataService, "Metadata service required");
		Validate.notNull(memberDetailsScanner,
				"Member details scanner required");
		Validate.notNull(vaadinEntityDetails, "Entity details required");

		this.vaadinEntityDetails = vaadinEntityDetails;
		this.useJpaContainer = useJpaContainer;

		this.metadataService = metadataService;
		this.memberDetailsScanner = memberDetailsScanner;
		this.webMetadataService = webMetadataService;
		this.typeLocationService = typeLocationService;
	}

	@Override
	public String toString() {
		ToStringBuilder tsb = new ToStringBuilder(this);
		tsb.append("identifier", getId());
		tsb.append("valid", valid);
		tsb.append("aspectName", aspectName);
		tsb.append("destinationType", destination);
		tsb.append("governor", governorPhysicalTypeMetadata.getId());
		tsb.append("itdTypeDetails", itdTypeDetails);
		return tsb.toString();
	}

	protected MetadataService getMetadataService() {
		return metadataService;
	}

	protected VaadinEntityViewMethodBuilderServices getServices() {
		return new VaadinEntityViewMethodBuilderServices(getMetadataService(),
				memberDetailsScanner, webMetadataService, typeLocationService);
	}

	protected VaadinEntityMetadataDetails getVaadinEntityDetails() {
		return vaadinEntityDetails;
	}

	protected JpaActiveRecordMetadata getEntityMetadata() {
		return vaadinEntityDetails.getEntityMetadata();
	}

	protected JavaType getEntityType() {
		return vaadinEntityDetails.getJavaType();
	}

	protected boolean isUseJpaContainer() {
		return useJpaContainer;
	}

	protected List<MethodMetadata> getDomainTypeCaptionMethods(
			Map<JavaSymbolName, JavaType> specialDomainTypes) {
		List<MethodMetadata> methods = new ArrayList<MethodMetadata>();

		DomainTypeCaptionMethodBuilder captionBuilder = new DomainTypeCaptionMethodBuilder(
				governorTypeDetails, getId(), vaadinEntityDetails,
				getServices(), builder, specialDomainTypes);

		for (MethodMetadata md : captionBuilder.getDomainTypeCaptionMethods()) {
			methods.add(md);
		}

		return methods;
	}

	protected List<MethodMetadata> getEntityMethods() {
		PersistenceMethodBuilder persistenceBuilder = new PersistenceMethodBuilder(
				governorTypeDetails, getId(), vaadinEntityDetails,
				getServices(), builder);

		return persistenceBuilder.getEntityMetadataMethods();
	}

	protected String getClassName(JavaType type) {
		return type.getNameIncludingTypeParameters(false,
				builder.getImportRegistrationResolver());
	}

}
