package com.vaadin.spring.roo.addon.entityform;

import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Service;
import org.osgi.service.component.ComponentContext;
import org.springframework.roo.addon.jpa.activerecord.JpaActiveRecordMetadata;
import org.springframework.roo.addon.web.mvc.controller.details.JavaTypeMetadataDetails;
import org.springframework.roo.classpath.PhysicalTypeIdentifier;
import org.springframework.roo.classpath.PhysicalTypeMetadata;
import org.springframework.roo.classpath.itd.ItdTypeDetailsProvidingMetadataItem;
import org.springframework.roo.classpath.scanner.MemberDetails;
import org.springframework.roo.model.JavaPackage;
import org.springframework.roo.model.JavaType;
import org.springframework.roo.project.LogicalPath;

import com.vaadin.spring.roo.addon.entityview.VaadinEntityMetadataDetails;

/**
 * Provides {@link VaadinAutomaticEntityFormMetadata}, creating ITD with form
 * content and field related methods.
 */
@Component(immediate = true)
@Service
public class VaadinAutomaticEntityFormMetadataProvider extends
		AbstractVaadinEntityFormMetadataProvider {

	protected void activate(ComponentContext context) {
		metadataDependencyRegistry.registerDependency(
				PhysicalTypeIdentifier.getMetadataIdentiferType(),
				getProvidesType());
		addMetadataTrigger(VaadinAutomaticEntityFormMetadata
				.getAnnotationType());
	}

	protected void deactivate(final ComponentContext context) {
		metadataDependencyRegistry.deregisterDependency(
				PhysicalTypeIdentifier.getMetadataIdentiferType(),
				getProvidesType());
		removeMetadataTrigger(VaadinAutomaticEntityFormMetadata
				.getAnnotationType());
	}

	@Override
	protected ItdTypeDetailsProvidingMetadataItem getMetadata(
			String metadataIdentificationString, JavaType aspectName,
			PhysicalTypeMetadata governorPhysicalTypeMetadata,
			String itdFilename) {

		// We know governor type details are non-null and can be safely cast

		// We need to parse the annotation, which we expect to be present
		VaadinAutomaticEntityFormAnnotationValues annotationValues = new VaadinAutomaticEntityFormAnnotationValues(
				governorPhysicalTypeMetadata);
		if (!annotationValues.isAnnotationFound()
				|| annotationValues.formBackingObject == null) {
			return null;
		}

		final MemberDetails memberDetails = getMemberDetails(governorPhysicalTypeMetadata);
		if (memberDetails == null) {
			return null;
		}

		return constructMetadata(metadataIdentificationString, aspectName,
				governorPhysicalTypeMetadata, itdFilename,
				annotationValues.formBackingObject);
	}

	@Override
	protected ItdTypeDetailsProvidingMetadataItem createMetadataItem(
			String metadataIdentificationString, JavaType aspectName,
			PhysicalTypeMetadata governorPhysicalTypeMetadata,
			JavaType formBackingObject, JpaActiveRecordMetadata entityMetadata,
			JavaPackage utilPackage, boolean useJpaContainer,
			JavaTypeMetadataDetails javaTypeMetadataDetails) {
		VaadinAutomaticEntityFormAnnotationValues annotationValues = new VaadinAutomaticEntityFormAnnotationValues(
				governorPhysicalTypeMetadata);
		return new VaadinAutomaticEntityFormMetadata(
				metadataIdentificationString, aspectName,
				governorPhysicalTypeMetadata, metadataService,
				memberDetailsScanner, getWebMetadataService(),
				typeLocationService, annotationValues,
				new VaadinEntityMetadataDetails(formBackingObject,
						entityMetadata, javaTypeMetadataDetails), utilPackage,
				useJpaContainer);
	}

	public String getItdUniquenessFilenameSuffix() {
		return "VaadinAutomaticEntityForm";
	}

	@Override
	protected String getGovernorPhysicalTypeIdentifier(
			String metadataIdentificationString) {
		JavaType javaType = VaadinAutomaticEntityFormMetadata
				.getJavaType(metadataIdentificationString);
		LogicalPath path = VaadinAutomaticEntityFormMetadata
				.getPath(metadataIdentificationString);
		String physicalTypeIdentifier = PhysicalTypeIdentifier
				.createIdentifier(javaType, path);
		return physicalTypeIdentifier;
	}

	@Override
	protected String createLocalIdentifier(JavaType javaType, LogicalPath path) {
		return VaadinAutomaticEntityFormMetadata.createIdentifier(javaType,
				path);
	}

	public String getProvidesType() {
		return VaadinAutomaticEntityFormMetadata.getMetadataIdentiferType();
	}

}
